/**
 * Account helper
 *
 * @author Pierre Hubert
 */

import { serverRequest } from "./APIHelper";

export interface AuthOptions {
	reset_token: string;
}

export interface AdminAccount {
	id: number;
	name: string;
	email: string;
	time_create: number;
}

const SESSION_STORAGE_TOKEN = "auth_token";

let currentAccount: AdminAccount | null = null;

export class AccountHelper {
	/**
	 * Check whether access token are available
	 * or not
	 */
	static get hasAccessToken(): boolean {
		return sessionStorage.getItem(SESSION_STORAGE_TOKEN) != null;
	}

	/**
	 * Get current user access token
	 */
	static get accessToken(): string {
		const token = sessionStorage.getItem(SESSION_STORAGE_TOKEN);
		if (!token) throw new Error("No access token for now!");
		return token;
	}

	/**
	 * Get authentication options for a given email address
	 *
	 * @param mail Requested email
	 */
	static async getAuthOptions(mail: string): Promise<AuthOptions> {
		return await serverRequest("accounts/auth_options", {
			mail: mail,
		});
	}

	/**
	 * Attempt to authenticate user using a reset token
	 *
	 * @param mail Email address of the admin
	 * @param token The reset token
	 */
	static async authWithResetToken(mail: string, token: string) {
		const res = await serverRequest("accounts/auth_with_reset_token", {
			mail: mail,
			token: token,
		});

		sessionStorage.setItem(SESSION_STORAGE_TOKEN, res.token);
	}

	/**
	 * Attempt to refresh current account information
	 */
	static async refreshCurrentAccountInfo() {
		const res = await serverRequest("accounts/info", {}, true);

		if (res.error) {
			if (res.error > 0) {
				sessionStorage.removeItem(SESSION_STORAGE_TOKEN);
				document.location.href = document.location.href + "";
			}
			throw new Error("Request failed!");
		}

		currentAccount = res;
	}

	/**
	 * Get current account information
	 */
	static get currentAccount(): AdminAccount {
		if (currentAccount == null) throw new Error("Current account is null!");
		return currentAccount;
	}

	/**
	 * Sign out current user
	 */
	static async signOut() {
		try {
			await serverRequest("accounts/sign_out");
		} catch (e) {
			console.error(e);
		}

		sessionStorage.removeItem(SESSION_STORAGE_TOKEN);
		document.location.href = document.location.href + "";
	}
}
